@@ -90,8 +90,9 @@ def cli():
 @click.argument("bucket")
 @click.argument("keys", nargs=-1)
 @click.option("--all", is_flag=True, help="Process all PDF files in the bucket")
+@click.option("--prefix", help="Process all PDF files within this prefix")
 @common_boto3_options
-def start(bucket, keys, all, **boto_options):
+def start(bucket, keys, all, prefix, **boto_options):
     """
     Start OCR tasks for PDF files in an S3 bucket
 
@@ -100,6 +101,10 @@ def start(bucket, keys, all, **boto_options):
     To process every file with a .pdf extension:
 
         s3-ocr start name-of-bucket --all
+
+    To process every .pdf in the PUBLIC/ folder:
+
+        s3-ocr start name-of-bucket --prefix PUBLIC/
     """
     s3 = make_client("s3", **boto_options)
     textract = make_client("textract", **boto_options)
@@ -114,11 +119,14 @@ def start(bucket, keys, all, **boto_options):
                 if match["Key"] in (key, key + S3_OCR_JSON):
                     items.append(match)
     else:
-        if not all:
+        if not all and not prefix:
             raise click.ClickException(
-                "Specify keys, or use --all to process all PDFs in the bucket"
+                "Specify keys, --prefix or use --all to process all PDFs in the bucket"
             )
-        items = list(paginate(s3, "list_objects_v2", "Contents", Bucket=bucket))
+        kwargs = dict(Bucket=bucket)
+        if prefix:
+            kwargs["Prefix"] = prefix
+        items = list(paginate(s3, "list_objects_v2", "Contents", **kwargs))
     # Start any item that ends in .pdf for which a .s3-ocr.json file does not exist
     keys_with_s3_ocr_files = [
         strip_ocr_json(item["Key"])
@@ -160,6 +168,81 @@ def start(bucket, keys, all, **boto_options):
                 click.echo(response)
 
 
+@cli.command
+@click.argument("bucket")
+@click.option(
+    "--dry-run", is_flag=True, help="Show output without writing anything to S3"
+)
+@common_boto3_options
+def dedupe(bucket, dry_run, **boto_options):
+    """
+    Scan every file in the bucket checking for duplicates - files that have
+    not yet been OCRd but that have the same contents (based on ETag) as a
+    file that HAS been OCRd.
+
+        s3-ocr dedupe name-of-bucket
+    """
+    s3 = make_client("s3", **boto_options)
+    click.echo("Scanning bucket {}".format(bucket), err=True)
+    items = list(paginate(s3, "list_objects_v2", "Contents", Bucket=bucket))
+    s3_ocr_to_fetch = [
+        (item["Key"], item["ETag"], strip_ocr_json(item["Key"]))
+        for item in items
+        if item["Key"].endswith(S3_OCR_JSON)
+    ]
+    keys_that_have_been_done = {s3_ocr[2] for s3_ocr in s3_ocr_to_fetch}
+
+    def _fetch():
+        for ocr_json_key, etag, key in s3_ocr_to_fetch:
+            response = s3.get_object(Bucket=bucket, Key=ocr_json_key)
+            data = json.loads(response["Body"].read())
+            yield {
+                "key": key,
+                "job_id": data["job_id"],
+                "etag": data["etag"],
+                "s3_ocr_etag": response["ETag"],
+            }
+
+    jobs_by_etag = {}
+    with click.progressbar(
+        _fetch(),
+        length=len(s3_ocr_to_fetch),
+        label="Fetching previous OCR jobs",
+        show_pos=True,
+    ) as rows:
+        for row in rows:
+            jobs_by_etag[row["etag"]] = row
+
+    # Check ETags of every file that has not been OCRd yet
+    not_yet_ocrd_keys = [
+        item
+        for item in items
+        if item["Key"].endswith(".pdf") and item["Key"] not in keys_that_have_been_done
+    ]
+
+    # Which of these are dupes?
+    dupes = {
+        item["Key"]: jobs_by_etag[item["ETag"]]
+        for item in not_yet_ocrd_keys
+        if item["ETag"] in jobs_by_etag
+    }
+
+    if dry_run:
+        click.echo("Would write results for the following dupes:")
+        click.echo(json.dumps(dupes, indent=2))
+    else:
+        with click.progressbar(
+            dupes.items(), label="Writing results for dupes", show_pos=True
+        ) as pairs:
+            for key, details in pairs:
+                body = {"job_id": details["job_id"], "etag": details["etag"]}
+                s3.put_object(
+                    Bucket=bucket,
+                    Key=f"{key}.s3-ocr.json",
+                    Body=json.dumps(body),
+                )
+
+
 @cli.command
 @click.argument("bucket")
 @common_boto3_options
@@ -184,6 +267,25 @@ def status(bucket, **boto_options):
     )
 
 
+@cli.command
+@click.argument("job_id")
+@common_boto3_options
+def inspect_job(job_id, **boto_options):
+    """
+    Show the current status of an OCR job
+
+        s3-ocr inspect-job <job_id>
+    """
+    textract = make_client("textract", **boto_options)
+    try:
+        response = textract.get_document_text_detection(JobId=job_id)
+    except textract.exceptions.InvalidJobIdException:
+        raise click.ClickException("Invalid job ID")
+    for key in ("Blocks", "ResponseMetadata", "NextToken"):
+        response.pop(key, None)
+    click.echo(json.dumps(response, indent=2))
+
+
 @cli.command
 @click.argument("bucket")
 @click.argument("key")
@@ -377,7 +479,7 @@ def index(bucket, database, **boto_options):
                     )
                 except StopIteration:
                     # This doesn't correspond to a job we know about
-                    print("Missing job ID:", job_id)
+                    click.echo("Missing job ID:", job_id, err=True)
                     continue
             path = job_row["key"]
             blocks = json.loads(
